   /* Wrapper for the second iteration step of DR regression at the central site
        Receive data from DPs and calculate various statistics. Determine if further iterations are necessary
        First version:  04/07/2016, Yury Vilk  
		Modified 20170130 (modification for job_done.ok/ job_fail.ok) 
   */
  %MACRO d_reg_central_step2;
      /* . After 20170130 (modification for job_done.ok job_fail) .*/
		%LOCAL prefix2 step_return_cd step_return_msg
                     _sscp_input_ds_sh addn_file_to_receive file_list_in tbl_list_in 
                     dp_list_with_missing_tbls dp_list_with_errors tmp_tbl
                      job_end_file_nm;

					 	/* Reset SAS automatic error variable.*/
   %LET SYSCC=0;

  %PUT;
   %PUT -----   Start of the macro d_reg_central_step2  iter_nb=&iter_nb.;
   %PUT;

	%LET step_nb=2;
    %LET step_return_cd=0;
   %LET step_return_msg=PASS;

	 %PUT;
	 %PUT Start of step: iter_nb=&iter_nb. step_nb=&step_nb. ;
	 %PUT;

 /*  %LET prefix2_prev=&prefix_prev.s&step_nb._; */
   %LET prefix2=&prefix.s&step_nb._;

   %PUT prefix=&prefix.;
   %PUT prefix2=&prefix2.;

  /* The program does not exit imediately when &runid_return_cd. > 0 because we want 
      to get tracking tables from DPs. They can have info about shutdown at each DP*/ 

	%IF &runid_return_cd. > 0 %THEN
	             %DO;
  %LET step_return_cd=11;
 %LET  step_return_msg=%STR(Error in the previous step. Current step  iter_nb=&iter_nb. step_nb=&step_nb..Run with RunID: &RunID. will terminate.) ;
	  %PUT;
	  %PUT  step_return_msg=&step_return_msg.;
	   %PUT;
				 %END;

/* Cleanup all files and SAS tables with names that will be created in the current step. */
  %delete_sas_tables(tbl_name_sh_list=&prefix2.:);
   %delete_sas_tables(lib=infolder,tbl_name_sh_list=&prefix2.:);
   %delete_sas_tables(lib=msoc,tbl_name_sh_list=&prefix2.:);
   %delete_sas_tables(lib=dplocal,tbl_name_sh_list=&prefix2.:);

 %watch_dp_msoc(dp_cd_list=&dp_cd_list.);

	  	 %IF &step_return_cd.=0 AND %LENGTH(&_dp_list_with_no_ok_file.) >0 %THEN  
										%DO;
	/* If wait time for file files_done.ok in one of msoc&dp_cd dir exceeded  wait_time_max*/

	%LET step_return_cd=2108;
   	%LET step_return_msg=%STR(Error:  Main program at the central site will stop executing. Wait time for file files_done.ok exceeded  wait_time_max=&wait_time_max. sec.  _dp_list_with_no_ok_file=&_dp_list_with_no_ok_file.);
    
	   	%PUT;
   		%PUT step_return_msg=&step_return_msg.;
   	    %PUT;
					  				    %END;

	      /* 2018020:  Added small delay to avoid file lock condition when deleting file   */

        DATA  _NULL_;	
		   CALL SLEEP(3,1);	
		 RUN;

  %initialize_tracking_tbl(dp_cd=0,tbl_name_out=&prefix2.tr_tb_updt1);

   %LET addn_file_to_receive=;
  %LET _sscp_input_ds_sh=&prefix.sscp.sas7bdat;

  			 /* For Cox regression */
	%IF &regr_type_cd. =10   %THEN
				        %DO;

 /* Note that the condinition &iter_nb. > %eval(&iter_nb_at_conv. + 1) can only be satisfied
      if &pos_dp_cd_in_strata_vars. =0 and thus &numb_iter_after_conv=2 */
			
	%IF	&iter_nb. <= %eval(&iter_nb_at_conv. + 1) %THEN 	 
		                               %DO;
   
           %IF &iter_nb. = 1  %THEN %LET addn_file_to_receive=&prefix0.total_cnts.sas7bdat;
			 %LET addn_file_to_receive=%SYSFUNC(COMPBL(&addn_file_to_receive. &prefix.sscp.sas7bdat));

	           %IF &pos_dp_cd_in_strata_vars. >0  %THEN
														%DO;

  %LET addn_file_to_receive=%SYSFUNC(COMPBL(&addn_file_to_receive. &prefix.resid_sum.sas7bdat));

 %IF &iter_nb. = 1  
 %THEN %LET addn_file_to_receive=%SYSFUNC(COMPBL(&addn_file_to_receive. &prefix0.mean_vars_by_st.sas7bdat));

%IF &iter_nb. = %eval(&iter_nb_at_conv. + 1) 
  %THEN	%LET addn_file_to_receive=%SYSFUNC(COMPBL(&addn_file_to_receive. &prefix0.baseln_hazard.sas7bdat 
                                                                                                 &prefix0.resid_sum_by_pct.sas7bdat));

														  %END;	 /* End of do-group  &pos_dp_cd_in_strata_vars. >0*/

					                   %END;  /* End of do-group  &iter_nb. <= %eval(&iter_nb_at_conv. + 1) */
		 %ELSE 	  
								       %DO;      
          %LET addn_file_to_receive=%SYSFUNC(COMPBL(&prefix0.resid_sum_by_pct.sas7bdat));
									   %END;   /* End of do-group &iter_nb. > %eval(&iter_nb_at_conv. + 1). Only possible when &pos_dp_cd_in_strata_vars. =0 */

						 %END;  /* End of do-group for &regr_type_cd. =10*/
		
						 /* Generalized Linear Model regression */
	   %ELSE 
			 %DO;
				    %IF    &regr_type_cd. =1  %THEN   
	                                        %DO;
	%IF &iter_nb. = 1  %THEN %LET addn_file_to_receive=&prefix.sscp.sas7bdat ;
	%ELSE 	 
                                                               %DO;
                          %LET addn_file_to_receive=&prefix.resid_sum.sas7bdat ;
						  %LET _sscp_input_ds_sh=;

	         		                                             %END; /* End of do-group &iter_nb.  NE  1  */
						                     %END;	/* End of do-group &regr_type_cd. =1   */
	 %ELSE 	    
	 										 %DO;
		%LET addn_file_to_receive=%SYSFUNC(COMPBL(&prefix.resid_sum.sas7bdat &prefix.sscp.sas7bdat)) ;
									      		%END;  /* End of do-group &regr_type_cd.  NOT IN (1 10) */

		%IF &iter_nb. = %eval(&iter_nb_at_conv. + 1) %THEN 
                     									%DO;
%LET addn_file_to_receive= %SYSFUNC(COMPBL(&addn_file_to_receive. &prefix0.sw_matrix.sas7bdat
                                        &prefix0.resid_sum_by_pct.sas7bdat	&prefix0.resid_sum_by_pct2.sas7bdat));
                      								     %END; 

			            %END;  	/* End of do-group &regr_type_cd.  NE 10 (GenMod)*/

	%LET file_list_in=%SYSFUNC(COMPBL(&addn_file_to_receive.  &prefix.tr_tb_updt.sas7bdat
                                                                                  &prefix.tr_tb_fl_updt.sas7bdat));

	 %LET tbl_list_in=%SYSFUNC(TRANSTRN(&file_list_in., .sas7bdat,));

 
		  /*Prepare table with file names coming from DPs. The actual directory will be defined in the next step*/

%output_file_list(tbl_list_not_transf=&file_list_in.
						    ,file_list_tbl_out=&prefix2.file_list_in0
							,send_received_cd=2
							);

DATA &prefix2.file_list_in	;
	  SET 	&prefix2.file_list_in0;
		 LENGTH dp_cd_from 4  dp_cd_list $ 1000 dir_nm_macro_nm $ 32;

		 dp_cd_list=SYMGET("dp_cd_list");
		 count_dp=COUNTW(STRIP(dp_cd_list));

			DO i=1 TO  count_dp;
			   dp_cd_from=INPUT(SCAN(STRIP(dp_cd_list),i) , 8.);
				dir_nm_macro_nm=CATS("msoc",dp_cd_from);
				dir_nm=SYMGET(dir_nm_macro_nm);
				full_file_name=CATS(dir_nm,file_nm);
				send_received_cd=2;

							  /*Defensive coding. Remove quotes if present*/
			IF         SUBSTR(dir_nm,1,1)=SUBSTR(dir_nm,LENGTH(dir_nm),LENGTH(dir_nm)) 
                AND SUBSTR(dir_nm,1,1) IN ("'",'"') 
			THEN  dir_nm=SUBSTR(dir_nm,2,LENGTH(dir_nm)-2);

			 IF    SUBSTR(dir_nm,LENGTH(dir_nm),LENGTH(dir_nm)) NOT IN ("/","\")
				 AND dir_nm NE " "
			 THEN  dir_nm=CATS(dir_nm,"/");

			dir_nm_sh=SCAN(STRIP(dir_nm),-1,"/\");
			OUTPUT	;
		   END;
	   DROP i  dp_cd_list  count_dp;
RUN;

  %find_files_mod_dtm( tbl_name_in=&prefix2.file_list_in , tbl_name_out=dplocal.&prefix2.file_list_in_dtm);

  %LET dp_list_with_missing_tbls=;

  %IF %LENGTH(&missing_or_0_obs_files.) >0 %THEN
         %DO;
	
PROC SQL NOPRINT;
  SELECT DISTINCT STRIP(PUT(dp_cd_from,8.)) INTO :dp_list_with_missing_tbls SEPARATED BY " "
    FROM dplocal.&prefix2.file_list_in_dtm
	   WHERE nobs <1
	   ;
  QUIT;

		%END;	/* End of do-group  missing_or_0_obs_files. */

 %IF           &step_return_cd. >0 OR  %LENGTH(&dp_list_with_missing_tbls.) >0 %THEN  
								%DO;
 %PUT;
 %PUT dp_list_with_missing_tbls=&dp_list_with_missing_tbls.;
 %PUT  missing_or_0_obs_files=&missing_or_0_obs_files.;
 %PUT Ristricting input files to be processed to tracking tables only;
  %PUT;

  %LET file_list_in=&prefix.tr_tb_updt.sas7bdat  &prefix.tr_tb_fl_updt.sas7bdat;
							   %END;


%PUT file_list_in=&file_list_in.;

  	/* Defensive coding. Initialize tracking table update that has info coming from  DPs. 
      This step is only necessary if no tracking tables came from DPs due to unexpected errors.
	 This should be rear. In most cases tracking table from DPs will be send back even 
     if there were errors. The actual table will be created in the next step.  
  */

		 	DATA &prefix.tr_tb_updt_c;
		      SET &prefix2.tr_tb_updt1(OBS=0);
			RUN;

		   DATA &prefix.tr_tb_fl_updt_c;
		      SET &prefix2.tr_tb_updt1(OBS=0 KEEP=DP_CD MSReqID RUNID ITER_NB STEP_NB);
			  LENGTH dir_nm_sh $ 300 file_nm $ 50;
				  dir_nm_sh=" ";
				  file_nm =" ";
			RUN;

			/* See note below about _i0_tr_tb_updt _i0_tr_tb_fl_updt  below*/ 
%combine_dp_tbls(dp_cd_list=&dp_cd_list.
		                           ,tbl_list=&tbl_list_in. _i0_tr_tb_updt _i0_tr_tb_fl_updt 
								   ,lib_out=
								   ,suffix=_c) ;

  PROC SORT DATA=&prefix.tr_tb_updt_c;
	    BY DP_CD MSReqID RUNID ITER_NB STEP_NB ;
RUN;


  PROC SORT DATA=&prefix.tr_tb_fl_updt_c;
	   BY DP_CD MSReqID RUNID ITER_NB STEP_NB DIR_NM_SH FILE_NM;
RUN;

     /* The tracking table can be trasfered without appropriate prefix only when DP does not 
		 receive parameter dataset or this dataset is unreadable. The code below deals with 
         such unlikely situation and tries to capture iinf coming from DP. 
     */
	%isdata(dataset= _i0_tr_tb_updt_c);
 
	 %IF &nobs. >0  %THEN
		                                  %DO;
			DATA _i0_tr_tb_updt_c;
			    SET _i0_tr_tb_updt_c;
			     RunID="&RunID.";
				  iter_nb=&iter_nb.;
				  step_nb=1;
			RUN;

	PROC SORT DATA=_i0_tr_tb_updt_c;
	    BY DP_CD MSReqID RUNID ITER_NB STEP_NB ;
    RUN;

	DATA &prefix.tr_tb_updt_c;
	  MERGE  _i0_tr_tb_updt_c &prefix.tr_tb_updt_c;
	    BY DP_CD MSReqID RUNID ITER_NB STEP_NB ;
	RUN;

	DATA _i0_tr_tb_fl_updt_c;
			    SET _i0_tr_tb_fl_updt_c;
			     RunID="&RunID.";
				  iter_nb=&iter_nb.;
				  step_nb=1;
			RUN;
	 
	   PROC SORT DATA=_i0_tr_tb_fl_updt_c;
	    BY DP_CD MSREQID RUNID ITER_NB STEP_NB DIR_NM_SH FILE_NM;
	   RUN;

	 DATA &prefix.tr_tb_fl_updt_c;
	   MERGE  _i0_tr_tb_fl_updt_c &prefix.tr_tb_fl_updt_c;
	    BY DP_CD MSReqID RUNID ITER_NB STEP_NB DIR_NM_SH FILE_NM;
	RUN;								
                                              %END;

	DATA dplocal.&prefix.tr_tb_updt_c;
      SET &prefix.tr_tb_updt_c;
    RUN;
  
 %LET dp_list_with_errors=;
	
PROC SQL NOPRINT;
  SELECT DISTINCT STRIP(PUT(dp_cd,8.)) INTO :dp_list_with_errors SEPARATED BY " "
    FROM dplocal.&prefix.tr_tb_updt_c
			 WHERE step_return_cd >0
	   ;
  QUIT;

  %PUT dp_list_with_errors=&dp_list_with_errors.;

      %IF&step_return_cd.=0 AND  %LENGTH(&dp_list_with_errors.) >0 %THEN 
 		 		 %DO;
 %LET  step_return_cd=3;	/* Errors in at least one of DPs  programss*/
%LET  step_return_msg=%STR(Error: Step fail iter_nb=&iter_nb. step_nb=&step_nb.. List of DPs with errors dp_list_with_errors:  &dp_list_with_errors..) ;
 %PUT;
 %PUT  step_return_msg=&step_return_msg.;
 %PUT;
               %END;

			        
 %IF &step_return_cd.=0 AND %LENGTH(&dp_list_with_missing_tbls.) >0 %THEN	 
				 				 %DO;
	%LET  step_return_cd=2;	/* Missing input files*/
	%LET  step_return_msg=%STR(Error: Step fail iter_nb=&iter_nb. step_nb=&step_nb.. List of DPs with missing tables dp_list_with_missing_tbls=&dp_list_with_missing_tbls..) ;
   %PUT;
   %PUT  step_return_msg=&step_return_msg.;
   %PUT;
								%END;

  %IF &step_return_cd.=0 AND  &SYSCC. > 4  %THEN	 
 				 %DO;
 %LET  step_return_cd=100;
 %LET  step_return_msg=%STR(ERROR: Step fail iter_nb=&iter_nb. step_nb=&step_nb. See log for details.);
  %PUT;
 %PUT  step_return_msg=&step_return_msg.;
 %PUT;
                %END;

       %IF &step_return_cd. >0 %THEN
                  %DO;
%IF &runid_return_cd.=0 %THEN %DO;
		  %LET runid_return_cd=&step_return_cd.;
		 %LET runid_return_msg=&step_return_msg.;
		                								 %END;
	     %GOTO endmac;  
				  %END;
  
				             /* Cox Model */
	%IF &regr_type_cd. = 10 %THEN
			%DO;
					      %IF &iter_nb. LE %eval(&iter_nb_at_conv. + 1) %THEN 
					                                 %DO; 
						               %calc_cox_iter_stats_central;
						                             %END; /*   End of do-group for &iter_nb. LE %eval(&iter_nb_at_conv. + 1)*/

					     %IF &iter_nb. > &iter_nb_at_conv.  %THEN
					                                            %DO;
						               %calc_final_stats_cox_central;
							                                   %END; 

				%END; /* End of do-group for &regr_type_cd. = 10 (Cox model) */

   %ELSE 
                             /* Generalized Linear Model*/
 			%DO;     
	                                 %calc_genmod_iter_stats_central;

	    %IF &iter_nb. > &iter_nb_at_conv.  %THEN
					                                            %DO;
 			                            %calc_final_stats_genmod_central;	
												               %END;  /*End of do-group for &iter_nb. > &iter_nb_at_conv.  */   
			%END; /* End of do-group for &regr_type_cd. NE 10 ( generalized linear model )*/


  %endmac:

         %IF &step_return_cd.=0 AND  &SYSCC. > 4  %THEN	 
 				 %DO;
 %LET  step_return_cd=100;
 %LET  step_return_msg=Step fail iter_nb=&iter_nb. step_nb=&step_nb.. See log for details;
  %PUT;
 %PUT  step_return_msg=&step_return_msg.;
 %PUT;
				%END;

 %IF &step_return_cd. >0 %THEN
                  %DO;
%IF &runid_return_cd.=0 %THEN %DO;
		  %LET runid_return_cd=&step_return_cd.;
		 %LET runid_return_msg=&step_return_msg.;
		                								 %END;
				  %END;
	
		 /*  For all iteration other the last_iter_in=1 the parameter  &track_tbl_csv_for_last_iter is blank 
			and no tracking table in csv format is outputed at this step (step_nb=2). Instead it is outputed
	       at the next iter_nb at the step_nb=1. For last_iter_in=1 the parameter  &track_tbl_csv_for_last_iter
	        is set to the &infolder.&prefix0.track_tbl.csv and the file is outputed.
	   */

				  
 %IF %LENGTH(&_dp_list_with_no_ok_file.) >0 %THEN
                  %DO;
			%LET last_iter_in=1;
			%LET last_runid_in=1;
			
			DATA dplocal.&prefix2.tr_tb_updt_c;
			  SET dplocal.&prefix2.tr_tb_updt_c;

			   last_iter_in=1;
			   last_runid_in=1;
			RUN;

				  %END;

	%IF  &last_iter_in.=1  %THEN  %LET track_tbl_csv_for_last_iter=&infolder.&prefix0.track_tbl.csv ;
   %ELSE 						                 %LET track_tbl_csv_for_last_iter=;

			 /* 20170119. Defensive coding */
%delete_ext_file(file_name=&infolder.&prefix0.track_tbl.csv, warning_flag=0);

%update_tracking_tbl(updt_tbl_name_in=&prefix2.tr_tb_updt1
                                      ,updt_tbl_name_c_in=dplocal.&prefix.tr_tb_updt_c
									  ,updt_tbl_name_out=dplocal.&prefix2.tr_tb_updt
									  ,tracking_tbl_out=infolder.&prefix0.track_tbl
									  ,file_list_txt_out=&track_tbl_csv_for_last_iter.
                                      );

  %create_tracking_tbl_fl_lev_updt(	updt_tbl_name_in=dplocal.&prefix2.tr_tb_updt
	                                                       ,file_list_dtm_in=dplocal.&prefix2.file_list_in_dtm
						                                   ,updt_tbl_name_fl_lev_out=dplocal.&prefix2.tr_tb_fl_updt);
													       
  %update_tracking_tbl_fl_lev(updt_tbl_name_fl_lev_in=dplocal.&prefix2.tr_tb_fl_updt
                                                 ,updt_tbl_name_fl_lev_c_in=&prefix.tr_tb_fl_updt_c
                                                  ,tracking_tbl_fl_lev_out=infolder.&prefix0.track_tbl_fl_lev);

%LET job_end_file_nm=;

	  %IF &last_iter_in.=1 %THEN 
		      %DO;
					   /* Cleanup before creating new versions (iteration level) */
  %delete_ext_file(file_name=&infolder.file_list.csv, warning_flag=0);
  %delete_ext_file(file_name=&infolder.files_done.ok, warning_flag=0);

 
		 %IF &last_runid_in.=1 %THEN 
		  								%DO;
		 /* Cleanup before creating new versions (job level) */
	   %delete_ext_file(file_name=&infolder.job_done.ok, warning_flag=0);
       %delete_ext_file(file_name=&infolder.job_fail.ok, warning_flag=0);

		%IF &step_return_cd.=0 %THEN 
					%DO;
	%LET job_end_file_nm=job_done.ok;
    %create_empty_file(file_nm=&infolder.job_done.ok);
	                   %END;
/*	  %ELSE											
						%DO;
	%LET job_end_file_nm=job_fail.ok;
    %create_empty_file(file_nm=&infolder.job_fail.ok);
	                      %END;
  */
										%END; /* End of do-group &last_runid_in.=1*/

				 /*Move outside do-group &last_runid_in.=1. Always exit if the there is an error */
		%IF &step_return_cd. NE 0 %THEN 
	 			%DO;
	%LET job_end_file_nm=job_fail.ok;
    %create_empty_file(file_nm=&infolder.job_fail.ok);
	                      %END;
	
   	%output_file_list(tbl_list_to_transf=
						    ,tbl_list_not_transf=&prefix0.track_tbl.csv  file_list.csv &job_end_file_nm.
						    ,file_list_txt_out=&infolder.file_list.csv
						    ,file_list_tbl_out=&prefix2.file_list
						    ,file_list_tbl_out_for_csv=dplocal.&prefix2.file_list_for_csv
							,dir_nm=&infolder.);
	
		 DATA  _NULL_;	
		   CALL SLEEP(5,1);	
		 RUN;

	 %create_empty_file(file_nm=&infolder.files_done.ok);

			  %END;	/* End of do-group &last_iter_in.=1 */					   

  	 %PUT;
	 %PUT End of step: iter_nb=&iter_nb. step_nb=&step_nb. ;
	 %PUT;
	 %PUT step_return_cd=&step_return_cd. ; 
     %PUT step_return_msg=&step_return_msg. ;
 	  %PUT;
      %PUT -----   End of the macro d_reg_central_step2;
      %PUT;

	%MEND;
