  /***************************************************************************************************
*                                           SENTINEL PROGRAM
****************************************************************************************************
* NAME:  run_d_reg_dp_templ
*
* PURPOSE: template for data partner master program for distributed regression;
*
*--------------------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yy): 08/20/16,  Yury Vilk.
*  Last modified date (mm/dd/yy): 02/23/18,  Yury Vilk.
*
**************************************************************************************************/

/*-------------------------------------------------------------------------------------------------
*                                        DEFINITIONS
*--------------------------------------------------------------------------------------------------
*
* MSReqID  ... Mini-Sentinel Project Request-Id
*              NOTE: This parameter needs to be set by the programmer preparing the Request-ID
*                    in coordination with the Mini-Sentinel Operations Center (MSOC)
*
* The following paths for the current request are automatically derived 
*    DPLOCAL  ... path for data created that remains local (i.e behind the DP firewall)
*    MSOC     ... path for data created to share with MS Operations Center
*    INFOLDER ... path for program inputs (e.g.lookup data, external macros)
*    SASPROGRAMS ... path for all SAS programs that are part of the data request
*    MACROS			   ... path for all SAS macros that are part of the data request
*-------------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* NOTE: Data Partner/Data Site programmer defines parameters in this section */
/*----------------------------------------------------------------------------*/


/* Edit data partner numeric code using lookup table. Maximum length 3 digits.*/

%LET dp_cd=1;

/***********************************************************************************
Data Partner Directory Macro Variables

IMPORTANT NOTES:
  1- DO use forward slashes (e.g. /) which are always compatible on both UNIX and WINDOWS
  2- DO use end of path separators (e.g. /xyz/ and not /xyz) which are assumed by many programs
  3- Do NOT surround directory paths with quotes (e.g. /xyz/ not "/xyz/") which will break the code
  
***********************************************************************************/

  * Edit root directory for  requests related to distrubuted regression. ;
 *  This is the directory into which you unziped the zip file with the current request.  ;

*%LET sites_prg_root_dp=\distrib_regr_all_requests_dp1\;
*%LET sites_prg_root_dp=J:\FDA_Sentinel_ASPE_2014\Internal testing\dp_sites\dp1\;
%LET sites_prg_root_dp=J:\prshare\MSCDM\PRG\Mod_Prg_Physical_MSCDM\_Dev\Distributed_Regression\ver1\dp_sites\dp1\;


/* Define request id for current request. 
   For Sentnel production one should comment/delete this line.
   The MSReqID will be defined at the central site before distributing request to 
   data partners. See statements below the line: "END OF USER INPUT" */


* %LET MSReqID = request_1;
 %LET MSReqID =v_glc_i01_pkg; 
 * %LET MSReqID =v_glc_i01_02_ln_lg; 
  *%LET MSReqID =v_glc_01;

  /* Edit directory which has the input regression dataset.
     Below is an example useful for testing. It refers to the subdirectory 
     within the same &MSReqID as the current request.

    For production it is likely to point to the sub-directory "dplocal" of the Sentinel request 
    which created analytic dataset. 
  */

  *%LET data_in_dir=&sites_prg_root_dp.MSReqID_for_step0\dplocal\;;
  %LET data_in_dir=&sites_prg_root_dp.&MSReqID./data_in/;

  /* Edit parameter min_count_per_grp to define minimum count per cell for summarized data 
   returned to central site. This affects datasets used for residual analysis and some 
   goodness of fit measures (ROC and HL statistic for logistic regression) */ 

 %LET min_count_per_grp=6;

**************************************************************************************************
*******                                  END OF USER INPUT                                  ******
*******                             DO NOT EDIT BELOW THIS LINE                             ******
*******                        (Consult with MSOC or MS team leader                         ******
*******                           if you feel edits are required)                           ******
**************************************************************************************************;

  /*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing Request-Id defines the                   */
/*       parameter MSReqID in coordination with the MSOC               */
/*---------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------------------
* Specific requests are made up a project-id, workplan-type, workplan-id, unique-dpid, version-id           
*--------------------------------------------------------------------------------------------------
*
*        If this is your request   . . . . .  then set parameter values as follows 
*   ---------------------------------     -------------------------------------------------------
*   [Task Order 7 - NME Dabigatran]       %let MSProjID = to07_nme_dabi ;  
*   [WorkPlan Type]                       %let MSWPType = ahr; 
*   [Workplan # 5]                        %let MSWPID = wp005 ; 
*   [Unique DP id]                        %let MSDPID = nsdp ;     
*   [Beta 3]                              %let MSVerID = b03 ;         
*   [use underscores as delimiter]        %let dlm = _ ;                
*
*   The example values above would produce a Request-Id equal to to07_nme_dabi_ahr_wp005_nsdp_b03
*
*--------------------------------------------------------------------------------------------------*/

* Specify project-id, workplan-type, worplan-id, and version-id *;
   /* Uncomment before sending to sites. The tags below are assigned by SOC*/
/*
%LET MSProjID = opi ;
%LET MSWPType = ahr;
%LET MSWPID = wp001 ;
%LET MSDPID = nsdp ;
%LET MSVerID = b01 ;
 
 %let dlm = _ ; 
 

* Request-Id *. 
 * MSReqID is expected by standard MSOC code below.;

 *%let MSReqID = &MSProjID.&dlm.&MSWPType.&dlm.&MSWPID.&dlm.&MSDPID.&dlm.&MSVerID ;
* Delete macro variable used as delimiter as we no longer need it *;
%symdel dlm ; 
  */


/* Request-Id/Workplan parameter value must be lower case */
%let MSReqID = %lowcase(&MSReqID) ;
*-------------------------------------------------------------------------------------------------
* 0- Setup Environment
*-------------------------------------------------------------------------------------------------;

/*---------------------------------------------------------------------*/
/* NOTE: This is standard MSOC environment setup code -- Do Not Edit   */
/*---------------------------------------------------------------------*/

/* System options */
options nosymbolgen nomlogic ;
options ls=100 nocenter ;
options obs=MAX ;
options nosyntaxcheck ;
options merror serror ;
options dkricond=error dkrocond=error;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options mergenoby= nowarn ;

/*------------------------------------------------------------------------------------------------
* NOTES:
*   For reasons of compatibility and standardization, paths must meet the following criteria:
*     1- Starting with the workplan subdirectory, all subdirectory names must be specified in
*        lower case (for UNIX compatibility)
*     2- Slashes must be forward-leaning slashes '/' (for UNIX compatibility)
*     3- Paths must end with a final forward-leaning slash '/' (extra slashes are fine)
*------------------------------------------------------------------------------------------------*/


%let _root_dplocal = &sites_prg_root_dp ;
%let _root_msoc =&sites_prg_root_dp ;
%let _root_inputfiles =&sites_prg_root_dp ;
%let _root_sasprograms =&sites_prg_root_dp;

/* Define Workplan Specific Subdirectories */
%let DPLOCAL = &_root_dplocal.&MSReqID./dplocal/ ; 
%let MSOC = &_root_msoc.&MSReqID./msoc/ ;            
%let INFOLDER = &_root_inputfiles.&MSReqID./inputfiles/ ;  
%let SASPROGRAMS = &_root_sasprograms.&MSReqID./sasprograms/ ;

*For Testing; 
*%LET SASMACR=&mod_prg_root.inputfiles/macros/; 
/*For Production*/
%let SASMACR=&infolder.macros/;

/* Assign libnames */
LIBNAME dplocal "&dplocal";
LIBNAME msoc "&msoc.";
LIBNAME infolder "&infolder.";
LIBNAME data_in "&data_in_dir.";

   %LET infolder_dir=&infolder.;
   	%LET infolder_lib=infolder;
	%LET msoc_dir=&msoc.;
	%LET msoc_lib=msoc;
	%LET dplocal_dir=&dplocal.;
   	%LET dplocal_lib=dplocal;
   	%LET data_in_lib=data_in;
	%LET exit_after_each_iter=0;

	   %PUT dp_cd=&dp_cd.;

**************************************************************************************************
*******                                  BEGIN PROGRAM                                     *******
**************************************************************************************************;

/*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing Workplan Request-ID includes/writes      */
/*       the rest of the program below                                 */
/*---------------------------------------------------------------------*/


  * Set up log file*;

  %LET prg_name_sh=&MSReqID.;

  %LET log_file=&msoc.&prg_name_sh..log;
  %LET lst_file=&dplocal.&prg_name_sh..lst;
      /*For DP*/
  * %LET lst_file=&dplocal.&prg_name_sh..lst;
 %PUT "&log_file.";

  %PUT "&lst_file.";

  * Set up log and list file if you don't want to use defaults*;
  /*
 proc printto log= "&log_file." new PRINT="&lst_file." new;;
run ;
 */


* Include macros ;
 	/* General utility macros for DP's and central */
   %include "&sasmacr.ms_macros_v2.sas"; 
  %include "&sasmacr.ms_time.sas";
  %include "&sasmacr.file_watcher.sas"; 
    %include "&sasmacr.check_var_list_in_tbl.sas";
    %include "&sasmacr.create_empty_file.sas"; 
   %include  "&sasmacr.delete_ext_file.sas" ;
   %include  "&sasmacr.delete_sas_tables.sas" ; 
   %include "&sasmacr.output_file_list.sas";
   %include "&sasmacr.find_files_mod_dtm.sas"; 
   %include "&sasmacr.calculate_sscp.sas";
   %include "&sasmacr.sum_by_pct2.sas";  
   %include "&sasmacr.export_sas_to_dlm_txt.sas";  
   %include "&sasmacr.import_dlm_txt_to_sas.sas"; 
    %include "&sasmacr.modify_colnames.sas"; 

   	  /*Distributed regression specific macros for DP */
   %include "&sasmacr.dp_main_vglc.sas";	 
 
             /*Genmod regression only (linear and logistic) macros*/
   %include "&sasmacr.calc_genmod_iter_stats_dp.sas"; 
   %include "&sasmacr.calc_genmod_vars.sas"; 
   %include "&sasmacr.calc_residual_stats_genmod_dp.sas";

                   /*Cox regression only  macros*/
   %include "&sasmacr.calc_cox_iter_stats_dp_vstr2_ef2.sas";	
    %include "&sasmacr.calc_cox_grad_hessian.sas";  
	%include "&sasmacr.calc_residual_stats_cox_dp_vstr2_ef2.sas";

				  /* Macros for initializing and updating of tracking tables */
	 %include "&sasmacr.initialize_tracking_tbl.sas"; 
	%include "&sasmacr.create_tracking_tbl_fl_lev_updt.sas"; 
	
*--------------------------------------------------------------------------------------------------
* 2- call macros
*--------------------------------------------------------------------------------------------------;

	   %dp_main(dp_cd=&dp_cd.);

	   /*    End  main wrapper  for the  DP package */
